﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Benday.FakesSample.DataAccess;

namespace Benday.FakesSample.Business
{
    public class PersonManager
    {
        public IPerson CreateNew()
        {
            return new Person(
                0, 
                String.Empty, 
                String.Empty, 
                String.Empty, 
                String.Empty, 
                new DateTime(1900, 1, 1), 
                new DateTime(1900, 1, 1));
        }

        public void Save(IPerson saveThis)
        {
            try
            {
                if (saveThis.Id == 0)
                {
                    SaveNew(saveThis);
                }
                else
                {
                    SaveExisting(saveThis);
                }
            }
            catch (SqlServerOutOfDiskSpaceException ex)
            {
                FixDiskSpaceProblem();
                new Logger().WriteToLog(ex);
                throw;
            }
            catch(Exception ex)
            {
                new Logger().WriteToLog(ex);
                throw;
            }
        }

        private void FixDiskSpaceProblem()
        {
            Console.WriteLine("Hey, somebody should really do something about that database.  It's outta disk space.");
        }

        private void SaveNew(IPerson saveThis)
        {
            PersonDataAccess dataAccess = new PersonDataAccess();

            saveThis.CreateDate = DateTime.Now;
            saveThis.LastModified = DateTime.Now;

            var datarow =
                new FakesDataSet().Person.AddPersonRow(
                    saveThis.FirstName,
                    saveThis.LastName,
                    saveThis.EmailAddress,
                    saveThis.Notes,
                    saveThis.CreateDate,
                    saveThis.LastModified);

            dataAccess.SavePerson(datarow);

            saveThis.Id = datarow.Id;
        }

        private static void SaveExisting(IPerson saveThis)
        {
            PersonDataAccess dataAccess = new PersonDataAccess();

            var datarow = dataAccess.LoadPerson(saveThis.Id);

            if (datarow == null)
            {
                throw new InvalidOperationException("Unknown person.");
            }

            saveThis.LastModified = DateTime.Now;

            datarow.FirstName = saveThis.FirstName;
            datarow.LastName = saveThis.LastName;

            if (datarow.EmailAddress != saveThis.EmailAddress)
            {
                datarow.EmailAddress = saveThis.EmailAddress;
            }

            datarow.Notes = saveThis.Notes;
            datarow.CreateDate = saveThis.CreateDate;
            datarow.LastModified = saveThis.LastModified;

            dataAccess.SavePerson(datarow);            
        }

        public IPerson LoadById(int id)
        {
            PersonDataAccess dataAccess = new PersonDataAccess();

            var datarow = dataAccess.LoadPerson(id);

            var person = new Person(
                datarow.Id, 
                datarow.FirstName, 
                datarow.LastName, 
                datarow.EmailAddress, 
                datarow.Notes, 
                datarow.CreateDate, 
                datarow.LastModified);

            return person;
        }
    }
}
