﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using Benday.FakesSample.Business;
using Benday.FakesSample.DataAccess;

namespace Benday.FakesSample.WinUI
{
    public partial class MainForm : Form
    {
        private PersonManager m_Manager;
        public PersonManager Manager
        {
            get
            {
                if (m_Manager == null)
                {
                    m_Manager = new PersonManager(new Logger());
                }

                return m_Manager;
            }
        }

        private IPerson m_Current;

        public MainForm()
        {
            InitializeComponent();
        }

        private void PopulateForm(IPerson fromValue)
        {
            m_textId.Text = fromValue.Id.ToString();
            m_textFirstName.Text = fromValue.FirstName;
            m_textLastName.Text = fromValue.LastName;
            m_textEmailAddress.Text = fromValue.EmailAddress;
            m_textNotes.Text = fromValue.Notes;
            m_textCreateDate.Text = fromValue.CreateDate.ToString();
            m_textLastModified.Text = fromValue.LastModified.ToString();
        }

        private void PopulateRow(IPerson fromValue)
        {
            fromValue.FirstName = m_textFirstName.Text.Trim();
            fromValue.LastName = m_textLastName.Text.Trim();
            fromValue.EmailAddress = m_textEmailAddress.Text.Trim();
            fromValue.Notes = m_textNotes.Text.Trim();            
        }

        private void LoadById()
        {
            int id = -1;
            bool isValidId = false;

            if (Int32.TryParse(m_textId.Text, out id) == true)
            {
                if (id > 0)
                {
                    isValidId = true;
                }
            }

            if (isValidId == false)
            {
                MessageBox.Show("Invalid id.");
            }
            else
            {
                m_Current = Manager.LoadById(id);

                if (m_Current == null)
                {
                    MessageBox.Show("Unknown record.");
                }
                else
                {
                    PopulateForm(m_Current);
                }
            }
        }

        private void m_btnLoad_Click(object sender, EventArgs e)
        {
            try
            {
                LoadById();
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.ToString());
            } 
        }

        private void Save()
        {
            if (m_Current == null)
            {
                MessageBox.Show("Nothing to save.");
            }
            else
            {
                PopulateRow(m_Current);
                Manager.Save(m_Current);
                PopulateForm(m_Current);
            }
        }

        private void m_btnSave_Click(object sender, EventArgs e)
        {
            try
            {
                Save();
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.ToString());
            } 
        }

        private void m_btnCreateNew_Click(object sender, EventArgs e)
        {
            CreateNew();
        }

        private void CreateNew()
        {
            m_Current = Manager.CreateNew();
            PopulateForm(m_Current);
        }
    }
}
